#!/usr/bin/env python
#Author: Mark Feineigle
#Create Date: 2017/06/23
import argparse
from collections import defaultdict, OrderedDict
from commands import getoutput
from datetime import datetime as dt
import json
import os
import sys

parser = argparse.ArgumentParser()
parser.add_argument("site", help="""site to parse = conected,
                                                    desertgo,
                                                    feineigle""")
parser.add_argument("-y", "--year", default=str(dt.now().year),
                                    help="""Year to parse.
                                            Default is this year.""")
parser.add_argument("-m", "--month", default=str(dt.now().month-1),
                                     help="""Month to parse (1-12). 
                                             Default is LAST month.""")
parser.add_argument("-a", "--all", action="store_true",
                                   help="Parse all months.")
args = parser.parse_args()

# scans the whole shebang of logs for a given site
def extract_all_logs(logPath):
  ips = []
  for filename in os.listdir(logPath):
    if filename.startswith("access"):
      if "gz" not in filename:
        ips.append(getoutput("cat "+logPath+filename+" | awk {'print $1 $4'}"))
      else:
        ips.append(getoutput("gunzip -c "+logPath+filename+" | awk {'print $1 $4'}"))
  # reformat the logs into a list:
  # ['2017-04-25 - 100.43.90.9', '2017-04-25 - 100.43.90.9', ...]
  ips = "\n".join(ips)
  ips = ips.split("\n")
  ips = [ip.split("[") for ip in ips if len(ip) > 1]
  ips = [dt.strftime(dt.strptime(date,"%d/%b/%Y:%H:%M:%S"),"%Y-%m-%d")+" - "+ip
        for ip, date in ips]
  return ips

def extract_month_from(month, ips):
  # sort out the month in question
  dd = defaultdict(set)
  tot_mo_hits = 0
  for line in ips:
    if line.startswith(year+"-"+month):
      tot_mo_hits += 1
      date , ip = line.strip().split(" - ")
      dd[date].add(ip)
  # order the month by day
  od = OrderedDict()
  for key, val in sorted(dd.items()):
    od[key] = list(val)
  data = json.dumps(od, indent=2)
  tot_mo_unique = sum([len(day) for day in od.values()])
  return (data, tot_mo_hits, tot_mo_unique)

def write_month_log(outPath, data, tot_mo_hits, tot_mo_unique):
  with open(outPath, "w") as fi:
    fi.write("Total hits: "+str(tot_mo_hits)+"\n")
    fi.write("Total unique hits: "+str(tot_mo_unique)+"\n")
    fi.write(data)

if __name__ == "__main__":
  site = args.site #desertgo, feineigle, conected
  year = args.year
  months = [args.month.zfill(2)] #pad with zeros if needed
  logPath = "/var/log/apache2/"+site+"/"
  archivePath = "/var/log/apache2/archive/"+site
  if not os.path.isdir(archivePath):
    os.mkdir(archivePath)

  # Loop for extraqcting all months at once (-a argument)
  if args.all == True:
    months = [str(m).zfill(2) for m in range(1,13)]
  for month in months:
    outPath = os.path.join(archivePath, site+"_"+year+"-"+month+".txt")
    ips = extract_all_logs(logPath)
    data, tot_mo_hits, tot_mo_unique = extract_month_from(month, ips)
    if len(data) > 2:
      write_month_log(outPath, data, tot_mo_hits, tot_mo_unique)
